"use strict";
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __metadata = (this && this.__metadata) || function (k, v) {
    if (typeof Reflect === "object" && typeof Reflect.metadata === "function") return Reflect.metadata(k, v);
};
var Core_1;
Object.defineProperty(exports, "__esModule", { value: true });
exports.Core = void 0;
/* eslint-disable @typescript-eslint/ban-ts-comment */
const di_1 = require("@ts-phoenix/di");
const event_manager_1 = require("@ts-phoenix/event-manager");
const container_1 = require("./container");
const defs_1 = require("./defs");
const errors_1 = require("./errors");
const events_1 = require("./events");
const tokens_1 = require("./tokens");
let Core = Core_1 = class Core {
    constructor(config) {
        this.config = config;
        this._state = defs_1.CoreState.NEW;
        this.eventManager = this.container.get(event_manager_1.EventManager);
        this.eventManager.addListener({
            event: events_1.CoreBeforeInitialiseEvent,
            handler: () => {
                this._state = defs_1.CoreState.INITIALIZING;
            },
        });
        this.eventManager.addListener({
            event: events_1.CoreAfterInitialiseEvent,
            handler: () => {
                this._state = defs_1.CoreState.INITIALIZED;
            },
        });
        this.setToken(tokens_1.CONTAINER, this.container);
        this.container.bind(Core_1).toConstantValue(this);
    }
    get state() {
        return this._state;
    }
    get isNew() {
        return this.state === defs_1.CoreState.NEW;
    }
    get isInitialised() {
        return this.state === defs_1.CoreState.INITIALIZED;
    }
    get isInitialising() {
        return this.state === defs_1.CoreState.INITIALIZING;
    }
    get container() {
        if (this._container == null) {
            this._container = new container_1.Container({
                autoBindInjectable: true,
                skipBaseClassChecks: true,
                defaultScope: di_1.BindingScopeEnum.Singleton,
            });
        }
        return this._container;
    }
    setToken(token, value) {
        this.container.bind(token.identifier).toConstantValue(value);
    }
    checkPackagesDependencies() {
        for (const pkg of this.config.packages) {
            const dependencies = pkg.getDependencies();
            for (const PackageDependency of dependencies) {
                if (!this.config.packages.some((pkg) => pkg instanceof PackageDependency)) {
                    throw new errors_1.DependencyNotFoundError({
                        dependency: PackageDependency.name,
                        dependent: pkg.constructor.name,
                    });
                }
            }
        }
    }
    async initialise() {
        if (!this.isNew) {
            return;
        }
        this.checkPackagesDependencies();
        for (const pkg of this.config.packages) {
            // @ts-ignore
            pkg.__setCore(this);
            // @ts-ignore
            pkg.__setConfigToken();
            if (!this.container.isBound(pkg.constructor)) {
                this.container.bind(pkg.constructor).toConstantValue(pkg);
            }
        }
        for (const pkg of this.config.packages) {
            await pkg.initialise();
        }
        await this.eventManager.emitAsync(new events_1.CoreBeforeInitialiseEvent());
        await this.eventManager.emitAsync(new events_1.CoreAfterInitialiseEvent());
    }
    async shutdown() {
        await this.eventManager.emitAsync(new events_1.CoreBeforeShutdownEvent());
    }
};
Core = Core_1 = __decorate([
    (0, di_1.Injectable)(),
    __metadata("design:paramtypes", [Object])
], Core);
exports.Core = Core;
